@@ -51,6 +51,7 @@
 #include "hints.h"
 #include "startup_notification.h"
 #include "events.h"
+#include "compositor.h"
 
 #define WIN_IS_BUTTON(win)      ((win == MYWINDOW_XWINDOW(c->buttons[HIDE_BUTTON])) || \
                                  (win == MYWINDOW_XWINDOW(c->buttons[CLOSE_BUTTON])) || \
@@ -1713,6 +1714,7 @@ handleEvent (DisplayInfo *display_info, XEvent * ev)
     TRACE ("entering handleEvent");
 
     sn_process_event (ev);
+    compositorHandleEvent (display_info, ev);
     switch (ev->type)
     {
         case MotionNotify:
