@@ -1,4 +1,5 @@
-/*
+/*      $Id: hints.c,v 1.85 2004/08/28 23:10:17 olivier Exp $
+ 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
@@ -15,7 +16,7 @@
  
         oroborus - (c) 2001 Ken Lynch
         Metacity - (c) 2001 Havoc Pennington
-        xfwm4    - (c) 2002-2003 Olivier Fourdan
+        xfwm4    - (c) 2002-2004 Olivier Fourdan
  
  */
 
@@ -23,28 +24,18 @@
 #include <config.h>
 #endif
 
-#ifdef GDK_MULTIHEAD_SAFE
-#undef GDK_MULTIHEAD_SAFE
-#endif
-
-#include <stdarg.h>
-#include <string.h>
-#include <stdio.h>
-#include <glib.h>
-#include <gtk/gtk.h>
-#include <gdk/gdk.h>
-#include <gdk/gdkx.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xmd.h>
-#include <libxfce4util/debug.h>
-#include <libxfce4util/i18n.h>
-#include <libxfce4util/utf8.h>
-#include <libxfcegui4/xinerama.h>
-#include <stdlib.h>
-#include <stdio.h>
+#include <glib.h>
+#include <gdk/gdk.h>
+#include <gdk/gdkx.h>
+#include <gtk/gtk.h>
+#include <stdarg.h>
 #include <string.h>
-#include "main.h"
+#include <stdio.h>
+#include <stdlib.h>
+#include <libxfce4util/libxfce4util.h> 
 #include "hints.h"
 
 Atom gnome_panel_desktop_area;
@@ -67,6 +58,7 @@ Atom wm_hints;
 Atom wm_protocols;
 Atom wm_state;
 Atom wm_takefocus;
+Atom wm_transient_for;
 Atom wm_window_role;
 
 /* freedesktop.org protocol */
@@ -77,7 +69,9 @@ Atom net_close_window;
 Atom net_current_desktop;
 Atom net_desktop_geometry;
 Atom net_desktop_viewport;
+Atom net_desktop_names;
 Atom net_number_of_desktops;
+Atom net_showing_desktop;
 Atom net_startup_id;
 Atom net_supported;
 Atom net_supporting_wm_check;
@@ -108,23 +102,29 @@ Atom net_wm_state_shaded;
 Atom net_wm_state_skip_pager;
 Atom net_wm_state_skip_taskbar;
 Atom net_wm_state_sticky;
+Atom net_wm_state_demands_attention;
 Atom net_wm_strut;
+Atom net_wm_strut_partial;
 Atom net_wm_window_type;
 Atom net_wm_window_type_desktop;
 Atom net_wm_window_type_dialog;
 Atom net_wm_window_type_dock;
 Atom net_wm_window_type_menu;
 Atom net_wm_window_type_normal;
-Atom net_wm_window_type_splashscreen;
+Atom net_wm_window_type_splash;
 Atom net_wm_window_type_toolbar;
 Atom net_wm_window_type_utility;
+Atom net_wm_user_time;
 Atom net_workarea;
 Atom utf8_string;
 
-/* Systray similation for older KDE apps */
+/* KDE extension */
+Atom kde_net_wm_context_help;
 Atom kde_net_wm_system_tray_window_for;
+Atom kwm_win_icon;
+
+/* Systray similation for older KDE apps */
 Atom net_system_tray_manager;
-Atom net_system_tray_selection;
 Atom net_system_tray_opcode;
 
 static gboolean
@@ -146,15 +146,26 @@ initICCCMHints (Display * dpy)
 {
     TRACE ("entering initICCCMHints");
 
-    sm_client_id = XInternAtom (dpy, "SM_CLIENT_ID", FALSE);
-    wm_change_state = XInternAtom (dpy, "WM_CHANGE_STATE", FALSE);
-    wm_client_leader = XInternAtom (dpy, "WM_CLIENT_LEADER", FALSE);
-    wm_colormap_windows = XInternAtom (dpy, "WM_COLORMAP_WINDOWS", FALSE);
-    wm_delete_window = XInternAtom (dpy, "WM_DELETE_WINDOW", FALSE);
-    wm_protocols = XInternAtom (dpy, "WM_PROTOCOLS", FALSE);
-    wm_state = XInternAtom (dpy, "WM_STATE", FALSE);
-    wm_takefocus = XInternAtom (dpy, "WM_TAKE_FOCUS", FALSE);
-    wm_window_role = XInternAtom (dpy, "WM_WINDOW_ROLE", FALSE);
+    sm_client_id = 
+        XInternAtom (dpy, "SM_CLIENT_ID", FALSE);
+    wm_change_state = 
+        XInternAtom (dpy, "WM_CHANGE_STATE", FALSE);
+    wm_client_leader = 
+        XInternAtom (dpy, "WM_CLIENT_LEADER", FALSE);
+    wm_colormap_windows = 
+        XInternAtom (dpy, "WM_COLORMAP_WINDOWS", FALSE);
+    wm_delete_window = 
+        XInternAtom (dpy, "WM_DELETE_WINDOW", FALSE);
+    wm_protocols = 
+        XInternAtom (dpy, "WM_PROTOCOLS", FALSE);
+    wm_state = 
+        XInternAtom (dpy, "WM_STATE", FALSE);
+    wm_takefocus = 
+        XInternAtom (dpy, "WM_TAKE_FOCUS", FALSE);
+    wm_window_role = 
+        XInternAtom (dpy, "WM_WINDOW_ROLE", FALSE);
+    wm_transient_for = 
+        XInternAtom (dpy, "WM_TRANSIENT_FOR", FALSE);
 }
 
 unsigned long
@@ -172,7 +183,10 @@ getWMState (Display * dpy, Window w)
                 (unsigned char **) &data) == Success) && (items_read))
     {
         state = *data;
-        XFree (data);
+        if (data)
+        {
+            XFree (data);
+        }
     }
     return state;
 }
@@ -219,7 +233,10 @@ getMotifHints (Display * dpy, Window w)
             result = g_new0(PropMwmHints, 1);
             memcpy (result, data, sizeof (PropMwmHints));
         }
-        XFree (data);
+        if (data)        
+        {
+            XFree (data);
+        }
     }
     return result;
 }
@@ -248,6 +265,11 @@ getWMProtocols (Display * dpy, Window w)
             {
                 result |= WM_PROTOCOLS_DELETE_WINDOW;
             }
+            /* KDE extension */
+            if (*ap == (Atom) kde_net_wm_context_help)
+            {
+                result |= WM_PROTOCOLS_CONTEXT_HELP;
+            }
         }
     }
     else
@@ -266,6 +288,11 @@ getWMProtocols (Display * dpy, Window w)
                 {
                     result |= WM_PROTOCOLS_DELETE_WINDOW;
                 }
+                /* KDE extension */
+                if (*ap == (Atom) kde_net_wm_context_help)
+                {
+                    result |= WM_PROTOCOLS_CONTEXT_HELP;
+                }
             }
         }
     }
@@ -284,22 +311,42 @@ initGnomeHints (Display * dpy)
 
     gnome_panel_desktop_area =
         XInternAtom (dpy, "GNOME_PANEL_DESKTOP_AREA", FALSE);
-    win_client_list = XInternAtom (dpy, "_WIN_CLIENT_LIST", FALSE);
+    win_client_list = 
+        XInternAtom (dpy, "_WIN_CLIENT_LIST", FALSE);
     win_desktop_button_proxy =
         XInternAtom (dpy, "_WIN_DESKTOP_BUTTON_PROXY", FALSE);
-    win_hints = XInternAtom (dpy, "_WIN_HINTS", FALSE);
-    win_layer = XInternAtom (dpy, "_WIN_LAYER", FALSE);
-    win_protocols = XInternAtom (dpy, "_WIN_PROTOCOLS", FALSE);
-    win_state = XInternAtom (dpy, "_WIN_STATE", FALSE);
+    win_hints = 
+        XInternAtom (dpy, "_WIN_HINTS", FALSE);
+    win_layer = 
+        XInternAtom (dpy, "_WIN_LAYER", FALSE);
+    win_protocols = 
+        XInternAtom (dpy, "_WIN_PROTOCOLS", FALSE);
+    win_state = 
+        XInternAtom (dpy, "_WIN_STATE", FALSE);
     win_supporting_wm_check =
         XInternAtom (dpy, "_WIN_SUPPORTING_WM_CHECK", FALSE);
-    win_workspace_count = XInternAtom (dpy, "_WIN_WORKSPACE_COUNT", FALSE);
-    win_workspace = XInternAtom (dpy, "_WIN_WORKSPACE", FALSE);
+    win_workspace_count = 
+        XInternAtom (dpy, "_WIN_WORKSPACE_COUNT", FALSE);
+    win_workspace = 
+        XInternAtom (dpy, "_WIN_WORKSPACE", FALSE);
 }
 
 void
+initKDEHints (Display * dpy)
+{
+    TRACE ("entering initKDEHints");
+
+    kde_net_wm_context_help = 
+        XInternAtom (dpy, "_NET_WM_CONTEXT_HELP", FALSE);
+    kde_net_wm_system_tray_window_for = 
+        XInternAtom (dpy, "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR", FALSE);
+    kwm_win_icon = XInternAtom (dpy, "KWM_WIN_ICON", FALSE);
+}
+
+Atom
 initSystrayHints (Display * dpy, int nscreen)
 {
+    Atom net_system_tray_selection;
     gchar selection[32];
 
     TRACE ("entering initSystrayHints");
@@ -308,8 +355,8 @@ initSystrayHints (Display * dpy, int nscreen)
     net_system_tray_manager   = XInternAtom (dpy, "MANAGER", FALSE);
     net_system_tray_opcode    = XInternAtom (dpy, "_NET_SYSTEM_TRAY_OPCODE", FALSE);
     net_system_tray_selection = XInternAtom (dpy, selection, FALSE);
-    kde_net_wm_system_tray_window_for = 
-        XInternAtom (dpy, "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR", FALSE);
+    
+    return net_system_tray_selection;
 }
 
 gboolean
@@ -330,7 +377,10 @@ getHint (Display * dpy, Window w, Atom a, long *value)
                 (unsigned char **) &data) == Success) && (items_read))
     {
         *value = *data;
-        XFree (data);
+        if (data)        
+        {
+            XFree (data);
+        }
         success = TRUE;
     }
     return success;
@@ -384,8 +434,7 @@ setGnomeProtocols (Display * dpy, int screen, Window w)
     XChangeProperty (dpy, RootWindow (dpy, screen), win_protocols, XA_ATOM,
         32, PropModeReplace, (unsigned char *) atoms, 1);
     setHint (dpy, w, win_supporting_wm_check, w);
-    setHint (dpy, RootWindow (dpy, screen), win_supporting_wm_check,
-        gnome_win);
+    setHint (dpy, RootWindow (dpy, screen), win_supporting_wm_check, w);
 }
 
 void
@@ -393,57 +442,92 @@ initNetHints (Display * dpy)
 {
     TRACE ("entering initNetHints");
 
-    net_active_window = XInternAtom (dpy, "_NET_ACTIVE_WINDOW", FALSE);
+    net_active_window = 
+        XInternAtom (dpy, "_NET_ACTIVE_WINDOW", FALSE);
     net_client_list_stacking =
         XInternAtom (dpy, "_NET_CLIENT_LIST_STACKING", FALSE);
-    net_client_list = XInternAtom (dpy, "_NET_CLIENT_LIST", FALSE);
-    net_close_window = XInternAtom (dpy, "_NET_CLOSE_WINDOW", FALSE);
-    net_current_desktop = XInternAtom (dpy, "_NET_CURRENT_DESKTOP", FALSE);
-    net_desktop_geometry = XInternAtom (dpy, "_NET_DESKTOP_GEOMETRY", FALSE);
-    net_desktop_viewport = XInternAtom (dpy, "_NET_DESKTOP_VIEWPORT", FALSE);
+    net_client_list = 
+        XInternAtom (dpy, "_NET_CLIENT_LIST", FALSE);
+    net_close_window = 
+        XInternAtom (dpy, "_NET_CLOSE_WINDOW", FALSE);
+    net_current_desktop = 
+        XInternAtom (dpy, "_NET_CURRENT_DESKTOP", FALSE);
+    net_desktop_geometry = 
+        XInternAtom (dpy, "_NET_DESKTOP_GEOMETRY", FALSE);
+    net_desktop_viewport = 
+        XInternAtom (dpy, "_NET_DESKTOP_VIEWPORT", FALSE);
+    net_desktop_names = 
+        XInternAtom (dpy, "_NET_DESKTOP_NAMES", FALSE);
     net_number_of_desktops =
         XInternAtom (dpy, "_NET_NUMBER_OF_DESKTOPS", FALSE);
-    net_startup_id = XInternAtom (dpy, "_NET_STARTUP_ID", FALSE);
-    net_supported = XInternAtom (dpy, "_NET_SUPPORTED", FALSE);
+    net_showing_desktop = 
+        XInternAtom (dpy, "_NET_SHOWING_DESKTOP", FALSE);
+    net_startup_id = 
+        XInternAtom (dpy, "_NET_STARTUP_ID", FALSE);
+    net_supported = 
+        XInternAtom (dpy, "_NET_SUPPORTED", FALSE);
     net_supporting_wm_check =
         XInternAtom (dpy, "_NET_SUPPORTING_WM_CHECK", FALSE);
     net_wm_action_change_desktop =
         XInternAtom (dpy, "_NET_WM_ACTION_CHANGE_DESKTOP", FALSE);
-    net_wm_action_close = XInternAtom (dpy, "_NET_WM_ACTION_CLOSE", FALSE);
+    net_wm_action_close = 
+        XInternAtom (dpy, "_NET_WM_ACTION_CLOSE", FALSE);
     net_wm_action_maximize_horz =
         XInternAtom (dpy, "_NET_WM_ACTION_MAXIMIZE_HORZ", FALSE);
     net_wm_action_maximize_vert =
         XInternAtom (dpy, "_NET_WM_ACTION_MAXIMIZE_VERT", FALSE);
-    net_wm_action_move = XInternAtom (dpy, "_NET_WM_ACTION_MOVE", FALSE);
-    net_wm_action_resize = XInternAtom (dpy, "_NET_WM_ACTION_RESIZE", FALSE);
-    net_wm_action_shade = XInternAtom (dpy, "_NET_WM_ACTION_SHADE", FALSE);
-    net_wm_action_stick = XInternAtom (dpy, "_NET_WM_ACTION_STICK", FALSE);
+    net_wm_action_move = 
+        XInternAtom (dpy, "_NET_WM_ACTION_MOVE", FALSE);
+    net_wm_action_resize = 
+        XInternAtom (dpy, "_NET_WM_ACTION_RESIZE", FALSE);
+    net_wm_action_shade = 
+        XInternAtom (dpy, "_NET_WM_ACTION_SHADE", FALSE);
+    net_wm_action_stick = 
+        XInternAtom (dpy, "_NET_WM_ACTION_STICK", FALSE);
     net_wm_allowed_actions =
         XInternAtom (dpy, "_NET_WM_ALLOWED_ACTIONS", FALSE);
-    net_wm_desktop = XInternAtom (dpy, "_NET_WM_DESKTOP", FALSE);
-    net_wm_icon_geometry = XInternAtom (dpy, "_NET_WM_ICON_GEOMETRY", FALSE);
-    net_wm_icon_name = XInternAtom (dpy, "_NET_WM_ICON_NAME", FALSE);
-    net_wm_icon = XInternAtom (dpy, "_NET_WM_ICON", FALSE);
-    net_wm_moveresize = XInternAtom (dpy, "_NET_WM_MOVERESIZE", FALSE);
-    net_wm_name = XInternAtom (dpy, "_NET_WM_NAME", FALSE);
-    net_wm_state_above = XInternAtom (dpy, "_NET_WM_STATE_ABOVE", FALSE);
-    net_wm_state_below = XInternAtom (dpy, "_NET_WM_STATE_BELOW", FALSE);
+    net_wm_desktop = 
+        XInternAtom (dpy, "_NET_WM_DESKTOP", FALSE);
+    net_wm_icon_geometry = 
+        XInternAtom (dpy, "_NET_WM_ICON_GEOMETRY", FALSE);
+    net_wm_icon_name = 
+        XInternAtom (dpy, "_NET_WM_ICON_NAME", FALSE);
+    net_wm_icon = 
+        XInternAtom (dpy, "_NET_WM_ICON", FALSE);
+    net_wm_moveresize = 
+        XInternAtom (dpy, "_NET_WM_MOVERESIZE", FALSE);
+    net_wm_name = 
+        XInternAtom (dpy, "_NET_WM_NAME", FALSE);
+    net_wm_state_above = 
+        XInternAtom (dpy, "_NET_WM_STATE_ABOVE", FALSE);
+    net_wm_state_below = 
+        XInternAtom (dpy, "_NET_WM_STATE_BELOW", FALSE);
     net_wm_state_fullscreen =
         XInternAtom (dpy, "_NET_WM_STATE_FULLSCREEN", FALSE);
-    net_wm_state_hidden = XInternAtom (dpy, "_NET_WM_STATE_HIDDEN", FALSE);
+    net_wm_state_hidden = 
+        XInternAtom (dpy, "_NET_WM_STATE_HIDDEN", FALSE);
     net_wm_state_maximized_horz =
         XInternAtom (dpy, "_NET_WM_STATE_MAXIMIZED_HORZ", FALSE);
     net_wm_state_maximized_vert =
         XInternAtom (dpy, "_NET_WM_STATE_MAXIMIZED_VERT", FALSE);
-    net_wm_state_modal = XInternAtom (dpy, "_NET_WM_STATE_MODAL", FALSE);
-    net_wm_state_shaded = XInternAtom (dpy, "_NET_WM_STATE_SHADED", FALSE);
+    net_wm_state_modal = 
+        XInternAtom (dpy, "_NET_WM_STATE_MODAL", FALSE);
+    net_wm_state_shaded = 
+        XInternAtom (dpy, "_NET_WM_STATE_SHADED", FALSE);
     net_wm_state_skip_pager =
         XInternAtom (dpy, "_NET_WM_STATE_SKIP_PAGER", FALSE);
     net_wm_state_skip_taskbar =
         XInternAtom (dpy, "_NET_WM_STATE_SKIP_TASKBAR", FALSE);
-    net_wm_state_sticky = XInternAtom (dpy, "_NET_WM_STATE_STICKY", FALSE);
-    net_wm_state = XInternAtom (dpy, "_NET_WM_STATE", FALSE);
-    net_wm_strut = XInternAtom (dpy, "_NET_WM_STRUT", FALSE);
+    net_wm_state_sticky = 
+        XInternAtom (dpy, "_NET_WM_STATE_STICKY", FALSE);
+    net_wm_state_demands_attention = 
+        XInternAtom (dpy, "_NET_WM_STATE_DEMANDS_ATTENTION", FALSE);
+    net_wm_state = 
+        XInternAtom (dpy, "_NET_WM_STATE", FALSE);
+    net_wm_strut = 
+        XInternAtom (dpy, "_NET_WM_STRUT", FALSE);
+    net_wm_strut_partial = 
+        XInternAtom (dpy, "_NET_WM_STRUT_PARTIAL", FALSE);
     net_wm_window_type_desktop =
         XInternAtom (dpy, "_NET_WM_WINDOW_TYPE_DESKTOP", FALSE);
     net_wm_window_type_dialog =
@@ -454,19 +538,24 @@ initNetHints (Display * dpy)
         XInternAtom (dpy, "_NET_WM_WINDOW_TYPE_MENU", FALSE);
     net_wm_window_type_normal =
         XInternAtom (dpy, "_NET_WM_WINDOW_TYPE_NORMAL", FALSE);
-    net_wm_window_type_splashscreen =
-        XInternAtom (dpy, "_NET_WM_WINDOW_TYPE_SPLASHSCREEN", FALSE);
+    net_wm_window_type_splash =
+        XInternAtom (dpy, "_NET_WM_WINDOW_TYPE_SPLASH", FALSE);
     net_wm_window_type_toolbar =
         XInternAtom (dpy, "_NET_WM_WINDOW_TYPE_TOOLBAR", FALSE);
     net_wm_window_type_utility =
         XInternAtom (dpy, "_NET_WM_WINDOW_TYPE_UTILITY", FALSE);
-    net_wm_window_type = XInternAtom (dpy, "_NET_WM_WINDOW_TYPE", FALSE);
-    net_workarea = XInternAtom (dpy, "_NET_WORKAREA", FALSE);
-    utf8_string = XInternAtom (dpy, "UTF8_STRING", FALSE);
+    net_wm_window_type = 
+        XInternAtom (dpy, "_NET_WM_WINDOW_TYPE", FALSE);
+    net_workarea = 
+        XInternAtom (dpy, "_NET_WORKAREA", FALSE);
+    net_wm_user_time = 
+        XInternAtom (dpy, "_NET_WM_USER_TIME", FALSE);
+    utf8_string = 
+        XInternAtom (dpy, "UTF8_STRING", FALSE);
 }
 
 void
-set_net_supported_hint (Display * dpy, int screen, Window check_win)
+setNetSupportedHint (Display * dpy, int screen, Window check_win)
 {
     Atom atoms[64];
     unsigned long data[1];
@@ -479,7 +568,9 @@ set_net_supported_hint (Display * dpy, int screen, Window check_win)
     atoms[i++] = net_current_desktop;
     atoms[i++] = net_desktop_geometry;
     atoms[i++] = net_desktop_viewport;
+    atoms[i++] = net_desktop_names;
     atoms[i++] = net_number_of_desktops;
+    atoms[i++] = net_showing_desktop;
     atoms[i++] = net_supported;
     atoms[i++] = net_supporting_wm_check;
     atoms[i++] = net_wm_action_change_desktop;
@@ -508,19 +599,21 @@ set_net_supported_hint (Display * dpy, int screen, Window check_win)
     atoms[i++] = net_wm_state_skip_pager;
     atoms[i++] = net_wm_state_skip_taskbar;
     atoms[i++] = net_wm_state_sticky;
+    atoms[i++] = net_wm_state_demands_attention;
     atoms[i++] = net_wm_strut;
+    atoms[i++] = net_wm_strut_partial;
     atoms[i++] = net_wm_window_type;
     atoms[i++] = net_wm_window_type_desktop;
     atoms[i++] = net_wm_window_type_dialog;
     atoms[i++] = net_wm_window_type_dock;
     atoms[i++] = net_wm_window_type_menu;
     atoms[i++] = net_wm_window_type_normal;
-    atoms[i++] = net_wm_window_type_splashscreen;
+    atoms[i++] = net_wm_window_type_splash;
     atoms[i++] = net_wm_window_type_toolbar;
     atoms[i++] = net_wm_window_type_utility;
+    atoms[i++] = net_wm_user_time;
     atoms[i++] = net_workarea;
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
-
     atoms[i++] = net_startup_id;
 #endif
 
@@ -534,7 +627,7 @@ set_net_supported_hint (Display * dpy, int screen, Window check_win)
 }
 
 gboolean
-get_atom_list (Display * dpy, Window w, Atom a, Atom ** atoms_p,
+getAtomList (Display * dpy, Window w, Atom a, Atom ** atoms_p,
     int *n_atoms_p)
 {
     Atom type;
@@ -571,7 +664,7 @@ get_atom_list (Display * dpy, Window w, Atom a, Atom ** atoms_p,
 }
 
 gboolean
-get_cardinal_list (Display * dpy, Window w, Atom xatom,
+getCardinalList (Display * dpy, Window w, Atom xatom,
     unsigned long **cardinals_p, int *n_cardinals_p)
 {
     Atom type;
@@ -604,25 +697,21 @@ get_cardinal_list (Display * dpy, Window w, Atom xatom,
 }
 
 void
-set_net_workarea (Display * dpy, int screen, int nb_workspaces, int * m)
+setNetWorkarea (Display * dpy, int screen, int nb_workspaces, int width, int height, int * m)
 {
     unsigned long *data, *ptr;
     int i, j;
 
-    TRACE ("entering set_net_workarea");
+    TRACE ("entering setNetWorkarea");
     j = (nb_workspaces ? nb_workspaces : 1);
     data = (unsigned long *) g_new0 (unsigned long, j * 4);
     ptr = data;
     for (i = 0; i < j; i++)
     {
-        *ptr++ = (unsigned long) m[MARGIN_LEFT];
-        *ptr++ = (unsigned long) m[MARGIN_TOP];
-        *ptr++ = (unsigned long) 
-            (MyDisplayFullWidth (dpy, screen) - 
-                (m[MARGIN_LEFT] + m[MARGIN_RIGHT]));
-        *ptr++ = (unsigned long) 
-            (MyDisplayFullHeight (dpy, screen) - 
-                (m[MARGIN_TOP] + m[MARGIN_BOTTOM]));
+        *ptr++ = (unsigned long) m[LEFT];
+        *ptr++ = (unsigned long) m[TOP];
+        *ptr++ = (unsigned long) (width  - (m[LEFT] + m[RIGHT]));
+        *ptr++ = (unsigned long) (height - (m[TOP] + m[BOTTOM]));
     }
     XChangeProperty (dpy, RootWindow (dpy, screen), net_workarea, XA_CARDINAL,
         32, PropModeReplace, (unsigned char *) data, j * 4);
@@ -630,12 +719,12 @@ set_net_workarea (Display * dpy, int screen, int nb_workspaces, int * m)
 }
 
 void
-init_net_desktop_params (Display * dpy, int screen, int workspace)
+initNetDesktopInfo (Display * dpy, int screen, int workspace, int width, int height)
 {
     unsigned long data[2];
-    TRACE ("entering init_net_desktop_params");
-    data[0] = MyDisplayFullWidth (dpy, screen);
-    data[1] = MyDisplayFullHeight (dpy, screen);
+    TRACE ("entering initNetDesktopInfo");
+    data[0] = width;
+    data[1] = height;
     XChangeProperty (dpy, RootWindow (dpy, screen), net_desktop_geometry,
         XA_CARDINAL, 32, PropModeReplace, (unsigned char *) data, 2);
     data[0] = 0;
@@ -684,7 +773,7 @@ getTransientFor (Display * dpy, int screen, Window w, Window * transient_for)
 }
 
 gboolean
-get_utf8_string (Display * dpy, Window w, Atom xatom, char **str_p)
+getUTF8String (Display * dpy, Window w, Atom xatom, char **str_p, int *length)
 {
     Atom type;
     int format;
@@ -692,7 +781,7 @@ get_utf8_string (Display * dpy, Window w, Atom xatom, char **str_p)
     unsigned char *str;
     unsigned long n_items;
 
-    TRACE ("entering get_utf8_string");
+    TRACE ("entering getUTF8String");
 
     *str_p = NULL;
     if ((XGetWindowProperty (dpy, w, xatom, 0, G_MAXLONG, FALSE, utf8_string,
@@ -703,8 +792,7 @@ get_utf8_string (Display * dpy, Window w, Atom xatom, char **str_p)
         return FALSE;
     }
 
-    if (!check_type_and_format (dpy, w, xatom, 8, utf8_string, -1, format,
-            type))
+    if (!check_type_and_format (dpy, w, xatom, 8, utf8_string, -1, format, type))
     {
         TRACE ("utf8_string value invalid");
         if (str)
@@ -721,9 +809,7 @@ get_utf8_string (Display * dpy, Window w, Atom xatom, char **str_p)
         name = XGetAtomName (dpy, xatom);
         if (name)
         {
-            TRACE
-                ("Property %s on window 0x%lx contains invalid UTF-8 characters",
-                name, w);
+            TRACE ("Property %s on window 0x%lx contains invalid UTF-8 characters", name, w);
             XFree (name);
         }
         XFree (str);
@@ -733,10 +819,11 @@ get_utf8_string (Display * dpy, Window w, Atom xatom, char **str_p)
 
     if (str)
     {
-        utf8_string_remove_controls((gchar *) str, -1, NULL);
+        xfce_utf8_remove_controls((gchar *) str, -1, NULL);
     }
     
     *str_p = str;
+    *length = n_items;
 
     return TRUE;
 }
@@ -752,9 +839,8 @@ text_property_to_utf8 (Display * dpy, const XTextProperty * prop)
 
     list = NULL;
     if ((count =
-            gdk_text_property_to_utf8_list (gdk_x11_xatom_to_atom (prop->
-                    encoding), prop->format, prop->value, prop->nitems,
-                &list)) == 0)
+            gdk_text_property_to_utf8_list (gdk_x11_xatom_to_atom (prop->encoding), 
+                             prop->format, prop->value, prop->nitems, &list)) == 0)
     {
         TRACE ("gdk_text_property_to_utf8_list returned 0");
         return NULL;
@@ -779,7 +865,7 @@ get_text_property (Display * dpy, Window w, Atom a)
         retval = text_property_to_utf8 (dpy, &text);
         if (retval)
         {
-            utf8_string_remove_controls((gchar *) retval, -1, NULL);
+            xfce_utf8_remove_controls((gchar *) retval, -1, NULL);
         }
         if ((text.value) && (text.nitems > 0))
         {
@@ -799,6 +885,7 @@ void
 getWindowName (Display * dpy, Window w, char **name)
 {
     char *str;
+    int len;
 
     TRACE ("entering getWindowName");
 
@@ -806,7 +893,7 @@ getWindowName (Display * dpy, Window w, char **name)
     *name = NULL;
     g_return_if_fail (w != None);
 
-    if (get_utf8_string (dpy, w, net_wm_name, &str))
+    if (getUTF8String (dpy, w, net_wm_name, &str, &len))
     {
         *name = strdup (str);
         XFree (str);
@@ -862,7 +949,7 @@ sendSystrayReqDock(Display * dpy, Window window, Window systray)
     xev.window = systray;
     xev.message_type = net_system_tray_opcode;
     xev.format = 32;
-    xev.data.l[0] = CurrentTime;
+    xev.data.l[0] = GDK_CURRENT_TIME;
     xev.data.l[1] = 0; /* SYSTEM_TRAY_REQUEST_DOCK */
     xev.data.l[2] = window;
     xev.data.l[3] = 0; /* Nada */
@@ -872,7 +959,7 @@ sendSystrayReqDock(Display * dpy, Window window, Window systray)
 }
 
 Window
-getSystrayWindow (Display * dpy)
+getSystrayWindow (Display * dpy, Atom net_system_tray_selection)
 {
     Window systray_win = None;
 
@@ -946,6 +1033,35 @@ getClientLeader (Display * dpy, Window window)
     return client_leader;
 }
 
+gboolean
+getNetWMUserTime (Display * dpy, Window window, Time *time)
+{
+    Atom actual_type;
+    int actual_format;
+    unsigned long nitems;
+    unsigned long bytes_after;
+    unsigned char *data = NULL;
+
+    TRACE ("entering getNetWMUserTime");
+
+    g_return_val_if_fail (window != None, None);
+
+    if (XGetWindowProperty (dpy, window, net_wm_user_time, 0L, 1L, FALSE,
+            XA_CARDINAL, &actual_type, &actual_format, &nitems,
+            &bytes_after, &data) == Success)
+    {
+        if ((data) && (actual_type == XA_CARDINAL)
+            && (nitems == 1) && (bytes_after == 0))
+        {
+            *time = *((long *) data);
+            XFree (data);
+            return TRUE;
+        }
+    }
+    *time = 0L;
+    return FALSE;
+}
+
 gboolean
 getClientID (Display * dpy, Window window, char **client_id)
 {
@@ -996,6 +1112,71 @@ getWindowCommand (Display * dpy, Window window, char ***argv, int *argc)
     return FALSE;
 }
 
+gboolean
+getKDEIcon (Display * dpy, Window window, Pixmap * pixmap, Pixmap * mask)
+{
+    Atom type;
+    int format;
+    unsigned long nitems;
+    unsigned long bytes_after;
+    Pixmap *icons;
+
+    *pixmap = None;
+    *mask = None;
+
+    icons = NULL;
+    if (XGetWindowProperty (dpy, window, kwm_win_icon, 0L, G_MAXLONG,
+            FALSE, kwm_win_icon, &type, &format, &nitems, &bytes_after,
+            (unsigned char **)&icons) != Success)
+    {
+        return FALSE;
+    }
+
+    if (type != kwm_win_icon)
+    {
+        if (icons)
+        {
+            XFree (icons);
+        }
+        return FALSE;
+    }
+
+    *pixmap = icons[0];
+    *mask = icons[1];
+
+    XFree (icons);
+
+    return TRUE;
+}
+
+gboolean
+getRGBIconData (Display * dpy, Window window, unsigned long **data, unsigned long *nitems)
+{
+    Atom type;
+    int format;
+    unsigned long bytes_after;
+
+    if (XGetWindowProperty (dpy, window, net_wm_icon, 0L, G_MAXLONG,
+                            FALSE, XA_CARDINAL, &type, &format, nitems,
+                            &bytes_after, (unsigned char **)data) != Success)
+    {
+        *data = NULL;
+        return FALSE;
+    }
+    
+    if (type != XA_CARDINAL)
+    {
+        if (*data)
+        {
+            XFree (*data);
+        }
+        *data = NULL;
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
 gboolean
 getWindowStartupId (Display * dpy, Window window, char **startup_id)
