@@ -1,4 +1,4 @@
-/*      $Id: main.c,v 1.109 2004/08/22 22:18:50 olivier Exp $
+/*      $Id: main.c,v 1.110 2004/09/09 22:54:17 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -57,7 +57,7 @@
 #include "startup_notification.h"
 #include "spinning_cursor.h"
 
-#define MAIN_EVENT_MASK\
+#define BASE_EVENT_MASK \
     SubstructureNotifyMask|\
     StructureNotifyMask|\
     SubstructureRedirectMask|\
@@ -67,6 +67,12 @@
     PropertyChangeMask|\
     ColormapNotify
 
+#ifdef HAVE_COMPOSITOR
+#define MAIN_EVENT_MASK BASE_EVENT_MASK|ExposureMask
+#else
+#define MAIN_EVENT_MASK BASE_EVENT_MASK
+#endif /* HAVE_COMPOSITOR */
+
 static DisplayInfo *display_info = NULL;
 gboolean xfwm4_quit           = FALSE;
 gboolean xfwm4_reload         = FALSE;
@@ -207,6 +213,14 @@ initialize (int argc, char **argv)
 
     display_info = myDisplayInit (gdk_display_get_default ());
 
+#ifdef HAVE_COMPOSITOR
+    if (!display_info->enable_compositor)
+    {
+        g_warning (_("%s: Cannot find required extensions " 
+                     "for compositing manager"), g_get_prgname ());
+    }
+#endif /* HAVE_COMPOSITOR */
+
     initModifiers (display_info->dpy);
 
     act.sa_handler = handleSignal;
